@@ -7,14 +7,52 @@ use base 'Object::Simple';
 
 use Carp 'croak';
 
-__PACKAGE__->attr([qw/default_filter filter filter_check filters sth/]);
+__PACKAGE__->attr([qw/filter_check filters sth/]);
+
+sub default_filter {
+    my $self = shift;
+    my $fname = $_[0];
+    
+    if (@_ && !$fname) {
+        $self->{_default_filter} = undef;
+    }
+    else {
+        croak qq{"$fname" is not registered}
+          unless exists $self->filters->{$fname};
+    
+        $self->{_default_filter} = $self->filters->{$fname};
+    }
+    
+    return $self;
+}
+
+sub filter {
+    my $self = shift;
+    my $filter = ref $_[0] eq 'HASH' ? $_[0] : {@_};
+    
+    foreach my $column (keys %$filter) {
+        my $fname = $filter->{$column};
+        unless (ref $fname eq 'CODE') {
+          croak qq{"$fname" is not registered"}
+            unless exists $self->filters->{$fname};
+          
+          $filter->{$column} = $self->filters->{$fname};
+        }
+    }
+    
+    $self->{_filter} = $filter;
+    
+    return $self;
+}
 
 sub fetch {
     my $self = shift;
     
     # Filters
     my $filters = $self->{filters} || {};
-    my $filter  = $self->{filter}  || {};
+    my $filter  = $self->{_filter}  || {};
+    my $auto_filter = $self->{_auto_filter} || {};
+    $filter = {%$auto_filter, %$filter};
     
     # Fetch
     my @row = $self->{sth}->fetchrow_array;
@@ -22,24 +60,18 @@ sub fetch {
     # No row
     return unless @row;
     
-    # Check filter
-    $self->_check_filter($filters, $filter, 
-                         $self->default_filter, $self->sth)
-      if $self->{filter_check};
-    
     # Filtering
     my $columns = $self->{sth}->{NAME_lc};
     for (my $i = 0; $i < @$columns; $i++) {
         
         # Filter name
         my $column = $columns->[$i];
-        my $fname  = exists $filter->{$column}
-                   ? $filter->{$column}
-                   : $self->{default_filter};
+        my $f  = exists $filter->{$column}
+               ? $filter->{$column}
+               : $self->{_default_filter};
         
         # Filtering
-        $row[$i] = $filters->{$fname}->($row[$i])
-          if $fname;
+        $row[$i] = $f->($row[$i]) if $f;
     }
 
     return \@row;
@@ -76,7 +108,9 @@ sub fetch_hash {
     
     # Filters
     my $filters = $self->{filters} || {};
-    my $filter  = $self->{filter}  || {};
+    my $filter  = $self->{_filter}  || {};
+    my $auto_filter = $self->{_auto_filter} || {};
+    $filter = {%$auto_filter, %$filter};
     
     # Fetch
     my $row = $self->{sth}->fetchrow_arrayref;
@@ -84,11 +118,6 @@ sub fetch_hash {
     # Cannot fetch
     return unless $row;
 
-    # Check filter
-    $self->_check_filter($filters, $filter, 
-                         $self->default_filter, $self->sth)
-      if $self->{filter_check};
-
     # Filter
     my $row_hash = {};
     my $columns = $self->{sth}->{NAME_lc};
@@ -96,14 +125,12 @@ sub fetch_hash {
         
         # Filter name
         my $column = $columns->[$i];
-        my $fname  = exists $filter->{$column}
-                   ? $filter->{$column}
-                   : $self->{default_filter};
+        my $f  = exists $filter->{$column}
+               ? $filter->{$column}
+               : $self->{_default_filter};
         
         # Filtering
-        $row_hash->{$column}
-          = $fname ? $filters->{$fname}->($row->[$i]) 
-                   : $row->[$i];
+        $row_hash->{$column} = $f ? $f->($row->[$i]) : $row->[$i];
     }
     
     return $row_hash;
@@ -174,30 +201,6 @@ sub fetch_multi {
     return $rows;
 }
 
-sub _check_filter {
-    my ($self, $filters, $filter, $default_filter, $sth) = @_;
-    
-    # Filter name not exists
-    foreach my $fname (values %$filter) {
-        croak qq{Fetch filter "$fname" is not registered}
-          unless exists $filters->{$fname};
-    }
-    
-    # Default filter name not exists
-    croak qq{Default fetch filter "$default_filter" is not registered}
-      if $default_filter && ! exists $filters->{$default_filter};
-    
-    # Column name not exists
-    my %columns = map {$_ => 1} @{$self->sth->{NAME_lc}};
-    foreach my $column (keys %$filter) {
-        croak qq{Column name "$column" in fetch filter must lower case string}
-          unless $column eq lc $column;
-        
-        croak qq{Column name "$column" in fetch filter is not found in result columns}
-          unless $columns{$column};
-    }
-}
-
 1;
 
 =head1 NAME
@@ -261,19 +264,6 @@ Fetch row into hash.
 
 =head1 ATTRIBUTES
 
-=head2 C<default_filter>
-
-    my $default_filter = $result->default_filter;
-    $result            = $result->default_filter('decode_utf8');
-
-Default filter when a row is fetched.
-
-=head2 C<filter>
-
-    my $filter = $result->filter;
-    $result    = $result->filter({title  => 'decode_utf8',
-                                  author => 'decode_utf8'});
-
 Filters when a row is fetched.
 This overwrites C<default_filter>.
 
@@ -303,6 +293,12 @@ Statement handle of L<DBI>.
 L<DBIx::Custom::Result> inherits all methods from L<Object::Simple>
 and implements the following new ones.
 
+=head2 C<(deprecated) default_filter>
+
+    $result = $result->default_filter($filter);
+
+Default filter when a row is fetched.
+
 =head2 C<fetch>
 
     my $row = $result->fetch;
@@ -353,4 +349,9 @@ Row count must be specified.
 Fetch multiple rows into array of array.
 Row count must be specified.
 
+=head2 C<filter>
+
+    $result    = $result->filter(title  => 'decode_utf8',
+                                 author => 'decode_utf8');
+
 =cut
